; ===============================================================================
; An initial Knowledge Base (KB) of a Controller Agent. Remember, that everything
; here is really only just __Belief__ (about the True State of the Environment)).
; ===============================================================================

; -------------------------------------------------------------------------
;(deftemplate ACLMessage 
;               (slot communicative-act) (slot sender) (multislot receiver)
;               (slot reply-with) (slot in-reply-to) (slot envelope)
;               (slot conversation-id) (slot protocol)
;               (slot language) (slot ontology) (slot content)
;               (slot encoding) (multislot reply-to) (slot reply-by))
;(deftemplate MyAgent (slot name))
;(deffunction send ())
; -------------------------------------------------------------------------

(deftemplate ValueOf (slot sensor-name) (slot sensor-type) (slot param-name) (slot param-value))

(deffacts times-of-the-day

    (time morning)
    (time noon)
    (time afternoon)
    (time evening)
    (time night)

)

(deffacts seasons-of-the-year

    (season spring)
    (season summer)
    (season autumn)
    (season winter)

)

(deffacts noise-levels

    (noise unnoticable)
    (noise noticable)
    (noise disturbing)

)

(deffacts temperature-levels

    (temperature cold)
    (temperature normal)
    (temperature hot)

)

(deffacts initial-facts

    (actual-season spring)
    (actual-time afternoon)
    (actual-noise noticable)
    (actual-temperature normal)
    
)

(defrule perceive-new-sensor-param

    (MyAgent (name ?me))
	?m <- (ACLMessage (communicative-act INFORM) (sender ?sn) (content ?pv) (receiver ?me) (language ?pn) (ontology ?st))
    (not (ValueOf (sensor-name ?sn) (sensor-type ?st) (param-name ?pn)))

	=>

    (retract ?m)
	(assert (ValueOf (sensor-name ?sn) (sensor-type ?st) (param-name ?pn) (param-value ?pv)))	; Create sensor-parameter-value representation

)

(defrule perceive-old-sensor-param

	(MyAgent (name ?me))
	?m <- (ACLMessage (communicative-act INFORM) (sender ?sn) (content ?pv) (receiver ?me) (language ?pn) (ontology ?st))
    ?f <- (ValueOf (sensor-name ?sn) (sensor-type ?st) (param-name ?pn))

	=>

    (retract ?m)
    (retract ?f)																				; Refresh sensor-parameter-value representation
	(assert (ValueOf (sensor-name ?sn) (sensor-type ?st) (param-name ?pn) (param-value ?pv)))	; Shouldn't use modify here, but assert, because...
																								; ...else the appropriate rules won't become active to be fired later.																								
)

(defrule some-reasoning

    (MyAgent (name ?s))
	(ValueOf (sensor-name ?r) (sensor-type music-volume-sensor) (param-name music-volume) (param-value "silent"))

	=>

    (assert (ACLMessage (communicative-act INFORM) (sender ?s) (receiver ?r) (content loud)))	; Notify the sensor (act toward the Environment)
    ;(printout t "The music volume measured by music sensor " ?r " is tried to be modified from silent to loud." crlf)

)

(defrule some-more-reasoning

	(ValueOf (sensor-type music-volume-sensor) (param-name music-volume) (param-value "loud"))
	(ValueOf (sensor-type window-sensor) (param-name window) (param-value "open"))
	(ValueOf (sensor-type ventillator-sensor) (param-name ventillator) (param-value "high"))
    ?f <- (actual-noise noticable)

	=>

	(retract ?f)
    (assert (actual-noise disturbing))			; This is not a action toward the environment, just the modification of the cognitive state (KB)

)

(defrule reasoning-again

    (MyAgent (name ?s))
	(ValueOf (sensor-name ?r) (sensor-type window-sensor) (param-name window) (param-value "open"))
	(actual-noise disturbing)
    
	=>

    (assert (ACLMessage (communicative-act INFORM) (sender ?s) (receiver ?r) (content closed)))	; Notify the sensor (act toward the Environment)

)

(defrule and-again

	(ValueOf (sensor-type music-volume-sensor) (param-name music-volume) (param-value "loud"))
	(ValueOf (sensor-type window-sensor) (param-name window) (param-value "closed"))
	(ValueOf (sensor-type ventillator-sensor) (param-name ventillator) (param-value "high"))
    ?f <- (actual-noise disturbing)

	=>

	(retract ?f)
    (assert (actual-noise noticable))			; This is not a action toward the environment, just the modification of the cognitive state (KB)

)

(defrule send-a-message

	(MyAgent (name ?me))
	?m <- (ACLMessage (sender ?me))

	=>

   	(send ?m)		; Send must surpass retracting!!
   	(retract ?m)

)

(watch facts)
(watch all)
(reset)