package milab.lab03.Environment.VentillatorAgent;

import jade.core.Agent;
import jade.core.behaviours.*;
import jade.lang.acl.ACLMessage;
import jade.lang.acl.MessageTemplate;
import jade.domain.DFService;
import jade.domain.FIPAException;
import jade.domain.FIPAAgentManagement.DFAgentDescription;
import jade.domain.FIPAAgentManagement.ServiceDescription;

public class VentillatorAgent extends Agent {
	
	// The music volume parameter observed/measured by the agent (silent, or soft, or normal, or loud)
	private String ventillator;
	// The GUI by means of which the user can externally modify the values of observed/measured parameters
	private VentillatorAgentGui myGui;
    	
	// Put agent initializations here
	protected void setup() {

		// Printout a welcome message
		System.out.println("Hello! Ventillator-agent " + getAID().getName() + " is ready");		
		
		// Initialize the parameters
		ventillator = "switched-of";
		
		// Create and show the GUI 
		myGui = new VentillatorAgentGui(this);
		myGui.show();

		// Register the service in the yellow pages
		DFAgentDescription dfd = new DFAgentDescription();
		dfd.setName(getAID());
		ServiceDescription sd = new ServiceDescription();
		sd.setType("general-sensor");
		sd.setName("ventillator-sensor");
		dfd.addServices(sd);
    
		try {
    	
			DFService.register(this, dfd);
      
		} catch (FIPAException fe) {
    	
			fe.printStackTrace();
      
		}

		// Add the behaviour serving queries from controller agents
		addBehaviour(new PerceptRequestServer());
		// Add the behaviour serving parameter modification from controller agents (i.e. the Environment)
		addBehaviour(new ParamControlServer());

	}

	// Put agent clean-up operations here
	protected void takeDown() {
  	
		// Deregister from the yellow pages
		try {
    	
			DFService.deregister(this);
      
		} catch (FIPAException fe) {
    	
			fe.printStackTrace();
      
		}
    
		// Close the GUI
		myGui.dispose();
		// Printout a dismissal message
		System.out.println("Ventillator-agent " + getAID().getName() + " terminating");
    
	}

	/*
		This is invoked by the GUI when the user modifies the musicVolume parameter (that is why it adds a behaviour: because it'll run in the agent's own thread unlike the GUI, which's events are executed in the eventDispatcher's thread...)
	*/
	public void updateVentillator(final String value) {

		addBehaviour(new OneShotBehaviour() {
    	
			public void action() {
      	
				// Updating the state of the ventillator
				ventillator = value;
				System.out.println("Ventillator updated to " + ventillator);
        
			}
      
		});
    
	}
    
	private class PerceptRequestServer extends CyclicBehaviour {
		
	  public void action() {
	  	
		  MessageTemplate mt = MessageTemplate.MatchPerformative(ACLMessage.REQUEST);
		  ACLMessage msg = myAgent.receive(mt);
		  
		  if (msg != null) {
	    	
			  // REQUEST Message received
			  ACLMessage reply = msg.createReply();
			  reply.setPerformative(ACLMessage.INFORM);																// Creating a universal Marshaller
			  reply.setContent(ventillator);
			  reply.setLanguage("ventillator");
			  reply.setOntology("ventillator-sensor");
			  myAgent.send(reply);
	      
		  } else {
	    	
			  block();
		    
		  }
		  
	  }
	  
	}  // End of inner class PerceptRequestServer

	private class ParamControlServer extends CyclicBehaviour {
		
		  public void action() {
		  	
			  MessageTemplate mt = MessageTemplate.MatchPerformative(ACLMessage.INFORM);
			  ACLMessage msg = myAgent.receive(mt);
			  
			  if (msg != null) {
		    	
				  ventillator = msg.getContent().toString();
				  myGui.setValueFieldText(ventillator);
		      
			  } else {
		    	
				  block();
			    
			  }
			  
		  }
		  
	}  // End of inner class ParamControlServer

	protected String getVentillator() {
		
		return ventillator;
		
	}

	protected void setVentillator(String value) {
		
		ventillator = value;
		
	}
	
}